#ifndef SEQUENCE_HH
#define SEQUENCE_HH

#include "base.hh"

#include <vector>
#include <string>

class Sequence;

class AliColumn: std::vector<char> {
    friend class Sequence;
public:
    double prob_equals(const AliColumn &c) const;
};


// Sequence is actually an alignment, since we want to align alignments, in the same way that we align sequences
// within the core algorithm, we want to almost forget about this fact

// access with operator [] is shifted!!!, i.e. indices 1..length (instead of 0..length-1)
class Sequence: std::vector<AliColumn> {
  
  int rows_;
  int len_;
  
  std::vector<std::string> names_;
  
public:    
  Sequence(int rows=0): rows_(rows), len_(0) {}; 
  
  // init as buffer with name and one row
  void init_buffer(const std::string &name);
    
  // init as buffer with the same names and number of rows as seq
  void init_buffer(const Sequence &seq);
  
  int length() const {return size();}

  int get_rows() const {return rows_;}

  void append_row(std::string name, std::string seqstr);
  
  AliColumn &operator [](int i)
  {return (std::vector<AliColumn>::operator [](i-1));}
  const AliColumn &operator [](int i) const
  {return (std::vector<AliColumn>::operator [](i-1));}

  void operator += (Sequence s);
  
  void operator += (AliColumn c);
  
  void operator += (char c);


  void write(std::ostream &out, int start, int end) const;

  void write(std::ostream &out) const;

};

// typedef string sequence_t;
typedef Sequence sequence_t;


#endif
